from __future__ import annotations

import functools

from typing import (
    Any,
    Tuple,
    Mapping,
    TypeVar,
    Callable,
    Sequence,
    cast,
)


_T = TypeVar("_T")
_TupleT = TypeVar("_TupleT", bound=Tuple[object, ...])
_MappingT = TypeVar("_MappingT", bound=Mapping[str, object])
_SequenceT = TypeVar("_SequenceT", bound=Sequence[object])
CallableT = TypeVar("CallableT", bound=Callable[..., Any])


def lru_cache(*, maxsize: int | None = 128) -> Callable[[CallableT], CallableT]:
    """A version of functools.lru_cache that retains the type signature
    for the wrapped function arguments.
    """
    wrapper = functools.lru_cache(  # noqa: TID251
        maxsize=maxsize,
    )
    return cast(Any, wrapper)  # type: ignore[no-any-return]



# -*- coding: UTF-8 -*-

from .node import Op
from .build import Where, SQL, SQLClient
from .app import App
from .tiefblue import Tiefblue

__all__ = [
    "Op", "Where", "SQL", "App", "SQLClient", "Tiefblue"
]



from __future__ import annotations

import time
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._client import OpenSDK


class SyncAPIResource:
    _client: OpenSDK

    def __init__(self, client: OpenSDK) -> None:
        self._client = client
        self._get = client.get
        self._post = client.post
        self._patch = client.patch
        self._put = client.put
        self._delete = client.delete

    def _sleep(self, seconds: float) -> None:
        time.sleep(seconds)



# -*- coding: UTF-8 -*-

from bohrium_open_sdk.db.node import Op, OperatorNode, Condition
from bohrium_open_sdk.db.app import App
import json


def Where(key: str, op: Op, value) -> OperatorNode:
    return OperatorNode(None, Condition.AND).And(key, op, value)


class SQLClient:
    def __init__(self, app_key, access_key=None, openapi_addr=None, timeout=60):
        self.app_key = app_key
        self.ak = access_key
        self._app = App(base_url=openapi_addr, access_key=access_key, app_key=app_key, timeout=timeout)
        self._app_id = ""

    def table(self, name):
        if name is None:
            raise ValueError("name not right")

        return SQL(name, self._app_id, self._app)

    def table_with_ak(self, table_ak):
        if table_ak == "":
            raise ValueError("table_ak not right")

        return SQL("", self._app_id, self._app, table_ak)


class SQL:
    def __init__(self, table, app_id, app: App, table_ak=""):
        self.table = table
        self.table_ak = table_ak
        self.root = None
        self._page = 1
        self._page_size = 10
        self._select = []
        self._data = []
        self._order = []
        self._app = app
        self._app_id = app_id

    def Select(self, *args):
        for key in args:
            self._select.append(key)
        return self

    def Where(self, key: str, op: Op, value):
        if self.root is None:
            self.root = Where(key, op, value)
        else:
            self.root.And(key, op, value)
        return self

    def Or(self, *args):
        if self.root is None:
            length = len(args)
            if length == 1:
                self.root = args[0]
            elif length == 3:
                self.root = Where(args[0], args[1], args[2])
            else:
                raise ValueError(f"The number of parameters is incorrect, must be 1 or 3, but got {length}")
        else:
            self.root.Or(*args)
        return self

    def And(self, *args):
        if self.root is None:
            length = len(args)
            if length == 1:
                self.root = args[0]
            elif length == 3:
                self.root = Where(args[0], args[1], args[2])
            else:
                raise ValueError(f"The number of parameters is incorrect, must be 1 or 3, but got {length}")
        else:
            self.root.And(*args)
        return self

    def page(self, count):
        self._page = count
        return self

    def page_size(self, count):
        self._page_size = count
        return self

    def order(self, key, is_asc: bool):
        o = -1
        if is_asc:
            o = 1
        order = {
            "filed": key,
            "type": o,
        }

        self._order.append(order)
        return self

    def Insert(self, obj):
        if type(obj) is dict:
            self._data.append(obj)
        elif type(obj) is list:
            self._data.extend(obj)
        else:
            raise ValueError("not support this type")

        json_data = {
            "appId": str(self._app_id),
            "tableName": self.table,
            "tableAk": self.table_ak,
            "data": self._data,
        }
        return self._app.insert(json_data)

    def Delete(self):
        filters = self._dict()
        if filters is None:
            raise ValueError("delete not support filter is none")

        json_data = {
            "appId": str(self._app_id),
            "tableAk": self.table_ak,
            "tableName": self.table,
            "filters": filters
        }
        return self._app.delete(json_data)

    def Count(self):
        filters = self._dict()
        json_data = {
            "appId": str(self._app_id),
            "tableAk": self.table_ak,
            "tableName": self.table,
            "filters": filters
        }
        return self._app.count(json_data)

    def Update(self, obj, upsert: bool = False):
        filters = self._dict()
        if filters is None:
            raise ValueError("update query is none")

        json_data = {
            "appId": str(self._app_id),
            "tableAk": self.table_ak,
            "tableName": self.table,
            "filters": filters,
            "values": obj,
            "options": {
                "upsert": upsert
            }
        }
        return self._app.update(json_data)

    def Find(self):
        filters = self._dict()
        json_data = {
            "appId": str(self._app_id),
            "tableAk": self.table_ak,
            "tableName": self.table,
            "selectedFields": [],
            "orderBy": [],
            "page": self._page,
            "pageSize": self._page_size,
        }

        if filters is not None:
            json_data["filters"] = filters

        if len(self._select) > 0:
            json_data["selectedFields"] = self._select

        if len(self._order) > 0:
            json_data["orderBy"] = self._order
        return self._app.query(json_data)

    def CreateTable(self, fields, index = None):
        if fields is None:
            raise ValueError("param error")

        data = {
            "appId": str(self._app_id),
            "tableName": self.table,
            "fields": fields,
        }

        if index is not None:
            data["index"] = index

        return self._app.create_table(data)

    def DeleteTable(self):
        data = {
            "appId": str(self._app_id),
            "tableName": self.table,
        }
        return self._app.delete_table(data)

    def _dict(self):
        if self.root is None:
            return None

        return self.root.dict()

    def _build(self):
        return json.dumps(self.root.dict(), indent=4)



import os

from setuptools import find_packages, setup


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename), encoding="utf-8") as f:
        long_description = f.read()
    return long_description


setup(
    name="bohrium-open-sdk",
    version="0.1.5-alpha.39",
    author="Bohrium Team",
    url="https://bohrium.dp.tech/developer",
    description="bohrium-open-sdk",
    long_description_content_type="text/markdown",  # file format
    packages=find_packages("src", exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    package_dir={"": "src"},  # find_packages define code directory
    package_data={
        # include .txt all of them
        "": ["*.txt"]
    },
    install_requires=[
        "httpx==0.27.0",
        "requests==2.32.3",
        "tqdm==4.66.4",
        "distro==1.9.0",
        "python-dotenv==1.0.0",
        "glom==23.5.0",
        "pytest==8.2.2",
    ],
    python_requires=">=3.8",
    entry_points={},
)
